//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DarkGray
#property indicator_color2  DarkGray
#property indicator_color3  DeepSkyBlue
#property indicator_color4  PaleVioletRed
#property indicator_width1  2
#property indicator_width3  2
#property indicator_width4  2
#property indicator_style2  STYLE_DOT
#property indicator_minimum 0
#property indicator_maximum 100

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    Length          = 32;
extern int    Smooth          =  3;
extern int    Signal          =  9;
extern int    SignalMode      = MODE_EMA;
extern int    Price           =  0;
extern double HighLowStep     = 0.5;
extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = true;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

double stoch[];
double signal[];
double max[];
double min[];
double trend[];
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
// 
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,stoch);
   SetIndexBuffer(1,signal);
   SetIndexBuffer(2,max);
   SetIndexBuffer(3,min);
   SetIndexBuffer(4,trend);
   timeFrame = stringToTimeFrame(TimeFrame);
      IndicatorShortName(timeFrameToString(timeFrame)+" Corridor stochastic ("+Length+","+Signal+","+Smooth+")");
   return(0);
}
int deinit() { return(0); }

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit=MathMax(Bars-counted_bars,3*timeFrame/Period());

   //
   //
   //
   //
   //

   for(int i=limit; i>=0; i--)
   {
      int y     = iBarShift(NULL,timeFrame,Time[i]);
      stoch[i]  = iStochastic(NULL,timeFrame,Length,Signal,Smooth,SignalMode,Price,MODE_MAIN  ,y);
      signal[i] = iStochastic(NULL,timeFrame,Length,Signal,Smooth,SignalMode,Price,MODE_SIGNAL,y);
      max[i]    = MathMax(stoch[i],max[i+1]-HighLowStep);
      min[i]    = MathMin(stoch[i],min[i+1]+HighLowStep);
      
      //
      //
      //
      //
      //
      
      trend[i] = trend[i+1];
         if (stoch[i]==max[i]) trend[i] =  1;
         if (stoch[i]==min[i]) trend[i] = -1;
   }         
   manageAlerts();
   return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar)); 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," "+timeFrameToString(timeFrame)+" corridor stochastic trend changed to "+doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" corridor stochastic",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}



